/*******************************************************************************
 *  This file does all of the event handling for the game.
 ******************************************************************************/
#ifndef UTIL_H
#include "util.h"
#endif

/*
--------------------------------------------------------------------------------
                                  EVENTS_MAIN
--------------------------------------------------------------------------------
*/
void events_main( SDL_Event &e )
{
    /*  Handle events */
    while( SDL_PollEvent( &e ) != 0 )
    {
        /*  If the user clicks the 'X' of the window (or similar) */
        if( e.type == SDL_QUIT )
            quit = true;

        /*  If the a joystick / gamepad stick has been moved UP/DOWN */
        else if( e.type == SDL_JOYAXISMOTION && e.jaxis.axis == 1 )
        {
            /*  Player 1's stick */
            if( e.jaxis.which == 0 )
                valuePad1 = e.jaxis.value;

            /*  Player 2's stick */
            else if( e.jaxis.which == 1 )
                valuePad2 = e.jaxis.value;
        }

        /*  If the user has released a joystick button */
        else if( e.type == SDL_JOYBUTTONUP )
        {
            /*  Which button was released */
            switch( e.button.which )
            {
                /*  START BUTTON - pause game or launch ball */
                case B_START:

                    /*
                     *  If a player has already won, we reset the scores, change
                     *  back to the main screen and reset the ball
                     */
                    if( gameOver )
                    {
                        gScores[ 0 ]->set_score( 0 );
                        gScores[ 1 ]->set_score( 0 );
                        currentScreen = SCREEN_MAIN;
                        gameOver = false;
                        paused = true;
                        reset = true;
                        pausedTicks = 0;
                    }

                    /*
                     *  Otherwise, if the game is just in its 'paused' state
                     *  (waiting for the ball to be launched) and a second has
                     *  passed, we launch the ball.
                     */
                    else if( paused && pausedTicks >= 60 )
                    {
                        paused = false;
                        pausedTicks = 0;
                        currentScreen = SCREEN_MAIN;

                        break;
                    }

                    /*  During gameplay, START behaves the same as SELECT */
                    else
                        /*  --  Fall through -- */

                /*  SELECT BUTTON - pause game */
                case B_SELECT:
                    /*  During gameplay, pause and bring up help screen */
                    if( currentScreen != SCREEN_HELP )
                    {
                        paused = true;
                        currentScreen = SCREEN_HELP;
                    }

                    /*  If we're on the help screen, return to main screen */
                    else
                    {
                        paused = false;
                        pausedTicks = 0;
                        currentScreen = SCREEN_MAIN;
                    }

                    break;

                /*  'A' BUTTON - change foreground color */
                case B_A:
                    change_color( fgColor );
                    gScores[ 0 ]->set_score( gScores[ 0 ]->get_score() );
                    gScores[ 1 ]->set_score( gScores[ 1 ]->get_score() );

                    /*  The game over screen requires special color cycling */
                    if( gameOver )
                    {
                        if( gScores[ 0 ]->get_score() >=
                                gScores[ 0 ]->get_max_score() )
                        {
                            reset_game_over_screen( 0 );
                        }
                        else
                            reset_game_over_screen( 1 );
                    }

                    break;

                /*  'B' BUTTON - change background color */
                case B_B:
                    change_color( bgColor );
                    gScores[ 0 ]->set_score( gScores[ 0 ]->get_score() );
                    gScores[ 1 ]->set_score( gScores[ 1 ]->get_score() );

                    /*  The game over screen requires special color cycling */
                    if( gameOver )
                    {
                        if( gScores[ 0 ]->get_score() >=
                                gScores[ 0 ]->get_max_score() )
                        {
                            reset_game_over_screen( 0 );
                        }
                        else
                            reset_game_over_screen( 1 );
                    }

                    break;

                /*  'X' BUTTON - toggle mute */
                case B_X:
                    /*  If the game isn't muted, mute it */
                    if( ! mute )
                    {
                        mute = true;
                        showOSD = true;
                        osd.update("SOUND OFF");
                    }

                    /*  If it is muted, unmute it */
                    else
                    {
                        mute = false;
                        showOSD = true;
                        osd.update("SOUND ON");
                    }

                    break;

                /*  'Y' BUTTON - Cycle AI paddle difficulty */
                case B_Y:
                    /*
                     *  Technically we cycle the difficulty for both paddles,
                     *  but since one player will always be controlling the left
                     *  paddle, in effect we're only cycling the difficulty for
                     *  the right paddle when it's set to AI
                     */
                    paddles[ 0 ]->increase_difficulty();
                    paddles[ 1 ]->increase_difficulty();

                    /*  Create the update string to inform the player */
                    char updateStr[ 32 ];
                    snprintf( updateStr, 32, "Difficulty:  %02d",
                            paddles[ 1 ]->get_difficulty() );
                    osd.update( updateStr );

                    showOSD = true;
                    break;

                /*  LEFT BUMPER - Toggle midline display */
                case B_LB:
                    if( midline )
                        midline = false;
                    else
                        midline = true;
                    break;

                /*  RIGHT BUMPER - Toggle score display */
                case B_RB:
                    if( showScores )
                        showScores = false;
                    else
                        showScores = true;
                    break;
            }

        }

        /*  If a keyboard key has been released */
        else if( e.type == SDL_KEYUP )
        {
            /*  We send the event to each paddle for paddle event handling */
            paddles[ 0 ]->handle_event( e );
            paddles[ 1 ]->handle_event( e );

            /*  Which key was released */
            switch( e.key.keysym.sym )
            {
                /*  ESCAPE KEY */
                case SDLK_ESCAPE:

                    /*  If we're on the help screen, return to main screen */
                    if( currentScreen == SCREEN_HELP )
                    {
                        paused = false;
                        currentScreen = SCREEN_MAIN;
                    }

                    /*  Otherwise, quit the program */
                    else
                        quit = true;

                    break;

                /*  Q KEY */
                case SDLK_q:

                    /*  Quit the program */
                    quit = true;
                    break;

                /*  SPACE KEY */
                case SDLK_SPACE:

                    /*  If a player has won, reset the game */
                    if( gameOver )
                    {
                        gScores[ 0 ]->set_score( 0 );   //  Reset scores
                        gScores[ 1 ]->set_score( 0 );
                        currentScreen = SCREEN_MAIN;    //  Go to main screen
                        gameOver = false;               //  No longer game over
                        paused = true;                  //  Pause game
                        reset = true;                   //  Reset ball
                        pausedTicks = 0;                //  Set pause timer to 0
                    }

                    /*  If the game is 'paused', launch the ball */
                    else if( pausedTicks >= 60 )
                    {
                        paused = false;
                        pausedTicks = 0;
                    }

                    break;

                /*  1 KEY - cycle foreground color */
                case SDLK_1:
                    change_color( fgColor );
                    gScores[ 0 ]->set_score( gScores[ 0 ]->get_score() );
                    gScores[ 1 ]->set_score( gScores[ 1 ]->get_score() );

                    /*  The game over screen requires special color cycling */
                    if( gameOver )
                    {
                        if( gScores[ 0 ]->get_score() >=
                                gScores[ 0 ]->get_max_score() )
                        {
                            reset_game_over_screen( 0 );
                        }
                        else
                            reset_game_over_screen( 1 );
                    }

                    break;

                /*  2 KEY - cycle background color */
                case SDLK_2:
                    change_color( bgColor );
                    gScores[ 0 ]->set_score( gScores[ 0 ]->get_score() );
                    gScores[ 1 ]->set_score( gScores[ 1 ]->get_score() );

                    /*  The game over screen requires special color cycling */
                    if( gameOver )
                    {
                        if( gScores[ 0 ]->get_score() >=
                                gScores[ 0 ]->get_max_score() )
                        {
                            reset_game_over_screen( 0 );
                        }
                        else
                            reset_game_over_screen( 1 );
                    }

                    break;

                /*  3 KEY - toggle midline display */
                case SDLK_3:
                    if( midline )
                        midline = false;
                    else
                        midline = true;
                    break;

                /*  4 KEY - toggle score display */
                case SDLK_4:
                    if( showScores )
                        showScores = false;
                    else
                        showScores = true;
                    break;

                /*  G KEY - grab / release mouse */
                case SDLK_g:
                    if( SDL_GetRelativeMouseMode() )
                        SDL_SetRelativeMouseMode( SDL_FALSE );
                    else
                        SDL_SetRelativeMouseMode( SDL_TRUE );
                    break;

                /*  H KEY - toggle help screen */
                case SDLK_h:
                    if( currentScreen != SCREEN_HELP )
                    {
                        paused = true;
                        currentScreen = SCREEN_HELP;
                    }
                    else
                    {
                        paused = false;
                        currentScreen = SCREEN_MAIN;
                    }
                    break;

                /*  S KEY - toggle sound on/off */
                case SDLK_s:
                    if( ! mute )
                    {
                        mute = true;
                        showOSD = true;
                        osd.update("SOUND OFF");
                    }

                    else
                    {
                        mute = false;
                        showOSD = true;
                        osd.update("SOUND ON");
                    }

                    break;

                /*  D KEY - cycle AI paddle difficulty */
                case SDLK_d:
                    paddles[ 0 ]->increase_difficulty();
                    paddles[ 1 ]->increase_difficulty();

                    char updateStr[ 32 ];
                    snprintf( updateStr, 32, "Difficulty:  %02d",
                            paddles[ 1 ]->get_difficulty() );
                    osd.update( updateStr );

                    showOSD = true;
                    break;

            }
        }

        /*  If the mouse has been moved */
        else if( e.type == SDL_MOUSEMOTION && ! paused )
        {
            /*  Send the event to the left paddle mouse event handler */
            paddles[ 0 ]->move_player_mouse( e );
        }
        
        /*  If the player has released a mouse button */
        else if( e.type == SDL_MOUSEBUTTONUP )
        {
            /*  Reset game if we're on the game over screen */
            if( gameOver )
            {
                gScores[ 0 ]->set_score( 0 );
                gScores[ 1 ]->set_score( 0 );
                currentScreen = SCREEN_MAIN;
                gameOver = false;
                paused = true;
                reset = true;
                pausedTicks = 0;
            }

            /*  Launch the ball, if applicable */
            else if( pausedTicks >= 60 )
            {
                paused = false;
                pausedTicks = 0;
            }
        }
    }
}
