#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                                    MOVEMENT
--------------------------------------------------------------------------------
*/

/*
 *  This function controls the player's paddle (left) with the keyboard
 */
void Paddle::move_player_keyboard( void )
{
    /*
     *  The velocity we use is the paddle's velocity halved so that movement
     *  isn't quite so dramatic
     */
    int vel = mVelY / 2;

    /*  Get the current state of the keyboard -- no delay, unlike event queue */
    const Uint8 *state = SDL_GetKeyboardState( NULL );

    /*  If the UP key is currently held */
    if( state[ SDL_SCANCODE_UP ] )
    {
        /*  Increase velocity */
        if( mVelY < mMaxVelocity )
            mVelY += 2;

        /*  Decrease Y position (move up) by velocity */
        mPos.y -= vel;
    }

    /*  If the DOWN key is currently held */
    if( state[ SDL_SCANCODE_DOWN ] )
    {
        /*  Increase velocity */
        if( mVelY < mMaxVelocity )
            mVelY += 2;

        /*  Increase Y position (move down) by velocity */
        mPos.y += vel;
    }
}


/*
 *  Allows player to move the paddle with the mouse
 */
void Paddle::move_player_mouse( SDL_Event &e )
{
    /*  Get the mouse position */
    int mx, my;
    SDL_GetMouseState( &mx, &my );

    /*  Set the middle of the paddle to the mouse position */
    mPos.y = ( my - (mHeight/2) );
}



/*
 *  This function lets the player move their paddle with the analog stick
 *  on a controller
 */
void Paddle::move_player_joystick( int padValue )
{
    int vel = mVelY / 2;

    /*  Moving up at a moderate pace */
    if( padValue < -(DEADZONE) && padValue > -( DEADZONE * 2 ) )
    {
        if( mVelY < mMaxVelocity )
            ++mVelY;

        mPos.y -= vel;
    }

    /*  Full speed up */
    else if( padValue < -( DEADZONE * 2 ) )
    {
        if( mVelY < mMaxVelocity )
            mVelY += 2;

        mPos.y -= vel;
    }

    /*  Down at a moderate pace */
    else if( padValue > DEADZONE && padValue < DEADZONE * 2 )
    {
        if( mVelY < mMaxVelocity )
            ++mVelY;

        mPos.y += vel;
    }

    /*  Full speed down */
    else if( padValue > DEADZONE * 2 )
    {
        if( mVelY < mMaxVelocity )
            mVelY += 2;

        mPos.y += vel;
    }

    /*  If no jaxis value, set Y velocity to zero */
    else
    {
        mVelY = 0;
    }


    /*
     *  Make sure the paddle is in bounds
     */

    if( mPos.y < 0 )
        mPos.y = 0;

    if( mPos.y > SCREEN_HEIGHT - mHeight )
        mPos.y = SCREEN_HEIGHT - mHeight;
}


/*
 *  Controls the AI paddle
 */
void Paddle::move_ai( void )
{
    /*  Increase number of ticks until next movement is allowed */
    ++moveTicks;

    /*  The higher the difficulty, the lower the delay between AI movements */
    if( moveTicks > ( 80 / mDifficulty ) )
    {
        /*  Middle of the AI paddle */
        int middle = mPos.y + ( mHeight / 2 );

        /*  If the center of the AI paddle matches the ball's Y position */
        if( middle == ball->get_pos_y() )
        {
            moveTicks = 0;
            mVelY = 0;
            dy = 0;
        }

        /*  If the middle of the paddle is higher than the ball, move down */
        else if( middle < ball->get_pos_y() )
        {
            /*  If the paddle is moving, increase velocity */
            if( dy > 0 )
            {
                ++mVelY;
                mPos.y += mVelY;
            }
            else
            {
                mVelY = 0;
                dy = 1;
            }

        }

        /*  If the middle of the paddle is lower than the ball, move up */
        else if( middle > ball->get_pos_y() + ball->get_height() )
        {
            if( dy < 0 )
            {
                ++mVelY;
                mPos.y -= mVelY;
            }
            else
            {
                mVelY = 0;
                dy = -1;
            }
        }
    }

}


/*
 *  Controls overall movement; allows for keyboard / AI movement and corrects
 *  movement for all movement methods
 */
void Paddle::move( void )
{
    /*  If the paddle is controlled by the player, react to player events */
    if( isAI )
        move_ai();
    else
        move_player_keyboard();


    /*
     *  Check to make sure the paddle is in bounds for keyboard / AI events
     */

    if( mPos.y < 0 )
        mPos.y = 0;

    if( mPos.y + mHeight > SCREEN_HEIGHT )
        mPos.y = SCREEN_HEIGHT - mHeight;


    /*  Reset the collider to the new position */
    mCollider.x = mPos.x;
    mCollider.y = mPos.y;

    /*  Handle collision */
    handle_collision();
}


/*
--------------------------------------------------------------------------------
                                  HANDLE_EVENT
--------------------------------------------------------------------------------
*/

/*
 *  Resets velocity if the up key or down key are released
 */
void Paddle::handle_event( SDL_Event &e )
{
    if( e.type == SDL_KEYUP )
    {
        if( e.key.keysym.sym == SDLK_UP || e.key.keysym.sym == SDLK_DOWN )
        {
            mVelY = 0;
        }
    }
}


/*
--------------------------------------------------------------------------------
                                   ACCESSORS
--------------------------------------------------------------------------------
*/


/*  Set the AI status of a paddle */
void Paddle::set_ai( bool status )
{
    isAI = status;
}

/*  Get the AI status of a paddle */
bool Paddle::is_ai( void )
{
    return( isAI );
}


/*  Set the player status of a paddle */
void Paddle::set_player( int playerNum )
{
    player = playerNum;
}

/*  Get the player status of a paddle */
int Paddle::get_player( void )
{
    return( player );
}


/*  Sets the difficulty of the AI paddle */
void Paddle::set_difficulty( int difficulty )
{
    mDifficulty = difficulty;
}

/*  Gets the difficulty of a paddle */
int Paddle::get_difficulty( void )
{
    return( mDifficulty );
}
